function [update]=AMANmodel(hOutput,hCancel,stimDef, trialDef, phaseDef, paramDef)
% RealTime Model of Associative Learning based on Harris & Livesey (2010)

% In contrast to the learning rule described in Harris & Livesey (2010,
% Eq.6), this version uses additionally the associability of the receiving
% unit to scale changes in associative learning. -> l.312 and so on

% Copyright (C) 2010  Anna Thorwart, Evan Livesey & Justin Harris, University of Sydney
% This file and instructions can be downloaded from http://sydney.edu.au/science/psychology/staff/justinh/downloads/


clc
update=get(hOutput,'String');
update=[update ;'Simulating ...'];
set(hOutput,'String',update, 'ListboxTop',length(update));drawnow;

%% Set random seed
%   generate a random stream from the computer clock
if verLessThan('matlab', '7.13')
    s = RandStream.create('mt19937ar','seed',sum(100*clock));
    RandStream.setDefaultStream(s);
    clear s
else
    rng('shuffle')
end;


%% create variables with values we will need often

num_stimulus_sources = size(stimDef,1);
num_total_elements = num_stimulus_sources*paramDef.num_elements;
num_phases=size(phaseDef,1);
num_TrialsPerPhase=sum(cell2mat(phaseDef(:,2)),2).* cell2mat(phaseDef(:,3));
num_overallTrials=sum(num_TrialsPerPhase);
num_Ticks=paramDef.num_TicksPerTrial*num_overallTrials;


%shared modalities of stimuli
modality_CS = cell2mat(stimDef(:,4));

%the range (min, max) of values of S for each stimulus source
%for now we'll set as follows...
S_range=cell2mat(stimDef(:,2:3));


%% Set up stimulus  and how their activate the elements

% Define activation  of  each elementof by each stimulus
S_from_stim = zeros(num_total_elements,num_stimulus_sources);

% Define array to keep track of same stimulus modalities for later normalisation
modality_check = zeros(num_total_elements,1);

count_pre = 0;
count_post = 0;
count_S = 0;

for ii = 1:num_stimulus_sources
    m1 = S_range(ii,1);
    m2 = S_range(ii,2);
    if paramDef.S_distribution == 1
        if isequal(m2,m1)
            randoms=ones(paramDef.num_elements,1)*m1;
        else
            %gradual steps from m2 down to m1
            mStep = (m2-m1)/(paramDef.num_elements-1);
            randoms = (m2:-mStep:m1);
        end
    elseif paramDef.S_distribution == 2
        %random values within the range m1 to m2
        mDiff = m2-m1;
        randoms = rand(1,paramDef.num_elements)*mDiff + m1;
    elseif paramDef.S_distribution == 3
        %all constant, reflecting mean of the two values m1 and m2
        randoms = repmat(mean([m1,m2]),1,paramDef.num_elements);
    else
        %no other method at the moment
    end
    count_post = count_pre + paramDef.num_elements;
    S_from_stim(count_pre+1:count_post,ii) = randoms; %context ii affects these elements
    %flag stimulus modality as well
    modality_check(count_pre+1:count_post) = modality_CS(ii);
    count_pre = count_post;
end

clear m1 m2 count_post count_pre count_S



%% Further representational architecture

%define various connections between elements
%these govern activation, learning and normalization

%random connectivity between E elements
connectivity_E_fromE = (rand(num_total_elements) < paramDef.connect_p);
%element E cannot form an association with itself
connectivity_E_fromE(1:length(connectivity_E_fromE)+1:numel(connectivity_E_fromE)) = 0;

weights_E_fromE = zeros(num_total_elements);
% weights_E_fromE = ones(num_total_elements).*.5;

weights_A_fromA = zeros(num_total_elements);
%buffer units are mutually inhibitory - all have same weights
weights_A_fromA(:,:) = paramDef.Weight_AA;
%A units don't inhibit themselves
weights_A_fromA(1:length(weights_A_fromA)+1:numel(weights_A_fromA)) = 0;

weights_E_fromI = zeros(num_total_elements);
%ii units always suppress their paired E unit
weights_E_fromI(1:length(weights_E_fromI)+1:numel(weights_E_fromI)) = 1;


% weights_A_fromE = zeros(num_total_elements);
% % A units receive mild inhibition from their paired E unit
% weights_A_fromE(ii,ii) = 0.0;


weights_I_fromA = zeros(num_total_elements);
%ii units receive strong inhibition from their paired A unit
weights_I_fromA(1:length(weights_I_fromA)+1:numel(weights_I_fromA)) = paramDef.Weight_IA;

%each E element has three corresponding activation values (for E, ii, & A units)...
activation_E = zeros(num_total_elements,1);
activation_I = zeros(num_total_elements,1);
activation_A = zeros(num_total_elements,1);

%...which are changing at these rates at any given instant
dE_dt = zeros(num_total_elements,1);
dI_dt = zeros(num_total_elements,1);
dA_dt = zeros(num_total_elements,1);

%the "weights_E_fromE" correspond to associative connections V,
%which have a rate of change and acceleration/decelaration
dV_dt = zeros(num_total_elements);
% d2V = zeros(num_total_elements);
% delta = zeros(num_total_elements);

%element associability Alpha also has a rate of change
Alpha = zeros(num_total_elements,1);
% dAlpha_dt = zeros(num_total_elements,1);

%% within-modality and within-stimulus normalisation (centre-surround like interactions)

%Determine element normalisation on basis of stimulus or modality
norm_stim = paramDef.norm_within_stim/paramDef.num_elements;
norm_modality = paramDef.norm_within_modality/paramDef.num_elements;

% normalsiation across all stimuli
weights_I_fromE = ones(num_total_elements)*paramDef.norm_across_all;


%within-modality normalization??
if norm_modality > 0
    % weights from other elements of same stim modality to I unit
    % elements that have the modality 0 are ingnored, ie. their weights
    % remain at the value of param.norm_across_all 
    for ii = 1:max(modality_CS)
        weights_I_fromE(modality_check==ii,modality_check==ii) = norm_modality;
    end
end


%within-stimulus normalization?
if norm_stim > 0
    %weak weights from other elements of same stimulus to I unit 
    count_pre = 0;
    count_post = 0;
    count_S = 0;
    
    for ii = 1:num_stimulus_sources
        count_post = count_pre + paramDef.num_elements;
        weights_I_fromE(count_pre+1:count_post,count_pre+1:count_post) = norm_stim;
        count_pre = count_post;
    end
    
end

%AND elements always activate their own I units 
weights_I_fromE(1:length(weights_I_fromE)+1:numel(weights_I_fromE)) = 1;

clear count_post count_pre count_S

%% Create the "real-world" events
%
events = zeros(num_stimulus_sources,num_Ticks);
trials = zeros(num_phases,max(num_TrialsPerPhase));
for pp=1:num_phases
    
    trials_in_phase=[];
    tmp=[];
    %find trial types presented in this phase
    for tt=find(phaseDef{pp,2})
        startIndex=sum(phaseDef{pp,2}(1:tt-1))+1;
        endIndex=sum(phaseDef{pp,2}(1:tt));
        tmp(startIndex:endIndex)=repmat(tt,phaseDef{pp,2}(tt),1);
    end
    
    for bb=1:phaseDef{pp,3}
       trials_in_phase=[trials_in_phase, tmp(randperm(length(tmp)))];
    end
    
    for tt=1:num_TrialsPerPhase(pp)
        for ss=find(trialDef{trials_in_phase(tt),3}-trialDef{trials_in_phase(tt),2}>0)
            startIndex=paramDef.num_TicksPerTrial*sum(num_TrialsPerPhase(1:pp-1))+ paramDef.num_TicksPerTrial*(tt-1)+trialDef{trials_in_phase(tt),2}(ss);
            endIndex=paramDef.num_TicksPerTrial*sum(num_TrialsPerPhase(1:pp-1))+paramDef.num_TicksPerTrial*(tt-1)+trialDef{trials_in_phase(tt),3}(ss);
            events(ss,startIndex:endIndex)=1;
        end
    end
    % to rearrange the output file, let's remember the trial order in each
    % phase
    trials(pp,1:length(trials_in_phase))=trials_in_phase;
end

clear trials_in_phase startIndex endIndex  pp tt ss

%% cycle through events, activate elements accordingly, perform weight
%% changes in realtime
rec_E = zeros(num_total_elements,num_Ticks);
rec_I = zeros(num_total_elements,num_Ticks);
rec_A = zeros(num_total_elements,num_Ticks);


for tick = 1:num_Ticks
    % work out actual activations for this step
    
    if mod(tick,5000)==0
        % check if user pushed the CANCEL button
        drawnow
        if get(hCancel,'Value')==1
            update=[update ;'Simulation canceled.'];
            set(hOutput,'String',update, 'ListboxTop',length(update));
            set(hCancel,'Value',get(hCancel, 'Min'));
            drawnow;
            return
        end
        
        update=[update ;[num2str(tick) ' of ' num2str(num_Ticks) ' time steps calculated.']]; %#ok<*AGROW>
        set(hOutput,'String',update, 'ListboxTop',length(update));drawnow;
        
    end
    
    
    current_S_from_stim = S_from_stim;
    S = current_S_from_stim * events(:,tick);
    
    % Runge-Kutta procedure for activations
    
    %k1
    actI = activation_I;
    actE = activation_E;
    actA = activation_A;
    alph = Alpha;
    V = weights_E_fromE;
    
    % inhibitory process
    Input_I = max(0,weights_I_fromE * actE);
    N_I = (weights_I_fromA * actA);
    I_pot = max(0,(Input_I.^paramDef.Power_I) ./ (Input_I.^paramDef.Power_I + N_I.^paramDef.Power_I + paramDef.D));
    diff_I = I_pot - actI;
    GD_I = paramDef.Growth_I .* (diff_I >= 0) + paramDef.Decay_I .* (diff_I < 0);
    k1_incI = GD_I .* diff_I;
    
    % excitatory process
    Input_E = max(0, S + (V * actE));
    N_E = weights_E_fromI * actI;
    E_pot = max(0,(Input_E.^paramDef.Power_E) ./ (Input_E.^paramDef.Power_E + N_E.^paramDef.Power_E + paramDef.D));
    diff_E = E_pot - actE;
    GD_E = paramDef.Growth_E .* (diff_E >= 0) + paramDef.Decay_E .* (diff_E < 0);
    k1_incE = GD_E .* diff_E;
    
    % attention process
    Input_A = max(0,S);
    %   N_A = (weights_A_fromA * actA)+ (weights_A_fromE * actE);
    N_A = (weights_A_fromA * actA);
    A_pot = max(0,(Input_A.^paramDef.Power_A) ./ (Input_A.^paramDef.Power_A + N_A.^paramDef.Power_A + paramDef.D));
    diff_A = A_pot - actA;
    GD_A = paramDef.Growth_A .* (diff_A >= 0) + paramDef.Decay_A .* (diff_A < 0);
    k1_incA = GD_A .* diff_A;
    
    % element associability
    k1_incAlpha = paramDef.k_Alpha * (actE - Alpha);
    
    % associative weight change
    Beta_E = paramDef.Beta_E_pos * (k1_incE >= 0) + paramDef.Beta_E_neg * (k1_incE < 0);
    Beta_I = paramDef.Beta_I_pos * (k1_incI >= 0) + paramDef.Beta_I_neg * (k1_incI < 0);
    diffV = ((Beta_E .* k1_incE) - (Beta_I .* k1_incI));
    k1_incV = dV_dt + (paramDef.k_V * ((((diffV.*alph)*alph').*connectivity_E_fromE) - dV_dt));
    
    
    %k2
    actI = activation_I + (0.5 * k1_incI);
    actE = activation_E + (0.5 * k1_incE);
    actA = activation_A + (0.5 * k1_incA);
    alph = Alpha + (0.5 * k1_incAlpha);
    V = weights_E_fromE + (0.5 * k1_incV);
    
    % inhibitory process
    Input_I = max(0,weights_I_fromE * actE);
    N_I = (weights_I_fromA * actA);
    I_pot = max(0,(Input_I.^paramDef.Power_I) ./ (Input_I.^paramDef.Power_I + N_I.^paramDef.Power_I + paramDef.D));
    diff_I = I_pot - actI;
    GD_I = paramDef.Growth_I .* (diff_I >= 0) + paramDef.Decay_I .* (diff_I < 0);
    k2_incI = GD_I .* diff_I;
    
    % excitatory process
    Input_E = max(0, S + (V * actE));
    N_E = weights_E_fromI * actI;
    E_pot = max(0,(Input_E.^paramDef.Power_E) ./ (Input_E.^paramDef.Power_E + N_E.^paramDef.Power_E + paramDef.D));
    diff_E = E_pot - actE;
    GD_E = paramDef.Growth_E .* (diff_E >= 0) + paramDef.Decay_E .* (diff_E < 0);
    k2_incE = GD_E .* diff_E;
    
    % attention process
    Input_A = max(0,S);
    %   N_A = (weights_A_fromA * actA)+ (weights_A_fromE * actE);
    N_A = (weights_A_fromA * actA);
    A_pot = max(0,(Input_A.^paramDef.Power_A) ./ (Input_A.^paramDef.Power_A + N_A.^paramDef.Power_A + paramDef.D));
    diff_A = A_pot - actA;
    GD_A = paramDef.Growth_A .* (diff_A >= 0) + paramDef.Decay_A .* (diff_A < 0);
    k2_incA = GD_A .* diff_A;
    
    % element associability
    k2_incAlpha = paramDef.k_Alpha * (actE - Alpha);
    
    % associative weight change
    Beta_E = paramDef.Beta_E_pos * (k2_incE >= 0) + paramDef.Beta_E_neg * (k2_incE < 0);
    Beta_I = paramDef.Beta_I_pos * (k2_incI >= 0) + paramDef.Beta_I_neg * (k2_incI < 0);
    diffV = ((Beta_E .* k2_incE) - (Beta_I .* k2_incI));
    k2_incV = dV_dt + (paramDef.k_V * ((((diffV.*alph)*alph').*connectivity_E_fromE) - dV_dt));
    
    
    %k3
    actI = activation_I + (0.5 * k2_incI);
    actE = activation_E + (0.5 * k2_incE);
    actA = activation_A + (0.5 * k2_incA);
    alph = Alpha + (0.5 * k2_incAlpha);
    V = weights_E_fromE + (0.5 * k2_incV);
    
    % inhibitory process
    Input_I = max(0,weights_I_fromE * actE);
    N_I = (weights_I_fromA * actA);
    I_pot = max(0,(Input_I.^paramDef.Power_I) ./ (Input_I.^paramDef.Power_I + N_I.^paramDef.Power_I + paramDef.D));
    diff_I = I_pot - actI;
    GD_I = paramDef.Growth_I .* (diff_I >= 0) + paramDef.Decay_I .* (diff_I < 0);
    k3_incI = GD_I .* diff_I;
    
    % excitatory process
    Input_E = max(0, S + (V * actE));
    N_E = weights_E_fromI * actI;
    E_pot = max(0,(Input_E.^paramDef.Power_E) ./ (Input_E.^paramDef.Power_E + N_E.^paramDef.Power_E + paramDef.D));
    diff_E = E_pot - actE;
    GD_E = paramDef.Growth_E .* (diff_E >= 0) + paramDef.Decay_E .* (diff_E < 0);
    k3_incE = GD_E .* diff_E;
    
    % attention process
    Input_A = max(0,S);
    %   N_A = (weights_A_fromA * actA)+ (weights_A_fromE * actE);
    N_A = (weights_A_fromA * actA);
    A_pot = max(0,(Input_A.^paramDef.Power_A) ./ (Input_A.^paramDef.Power_A + N_A.^paramDef.Power_A + paramDef.D));
    diff_A = A_pot - actA;
    GD_A = paramDef.Growth_A .* (diff_A >= 0) + paramDef.Decay_A .* (diff_A < 0);
    k3_incA = GD_A .* diff_A;
    
    % element associability
    k3_incAlpha = paramDef.k_Alpha * (actE - Alpha);
    
    % associative weight change
    Beta_E = paramDef.Beta_E_pos * (k3_incE >= 0) + paramDef.Beta_E_neg * (k3_incE < 0);
    Beta_I = paramDef.Beta_I_pos * (k3_incI >= 0) + paramDef.Beta_I_neg * (k3_incI < 0);
    diffV = ((Beta_E .* k3_incE) - (Beta_I .* k3_incI));
    k3_incV = dV_dt + (paramDef.k_V * ((((diffV.*alph)*alph').*connectivity_E_fromE) - dV_dt)); %todo
    
    
    %k4
    actI = activation_I + (1 * k3_incI);
    actE = activation_E + (1 * k3_incE);
    actA = activation_A + (1 * k3_incA);
    alph = Alpha + (1 * k3_incAlpha);
    V = weights_E_fromE + (1 * k3_incV);
    
    % inhibitory process
    Input_I = max(0,weights_I_fromE * actE);
    N_I = (weights_I_fromA * actA);
    I_pot = max(0,(Input_I.^paramDef.Power_I) ./ (Input_I.^paramDef.Power_I + N_I.^paramDef.Power_I + paramDef.D));
    diff_I = I_pot - actI;
    GD_I = paramDef.Growth_I .* (diff_I >= 0) + paramDef.Decay_I .* (diff_I < 0);
    k4_incI = GD_I .* diff_I;
    
    % excitatory process
    Input_E = max(0, S + (V * actE));
    N_E = weights_E_fromI * actI;
    E_pot = max(0,(Input_E.^paramDef.Power_E) ./ (Input_E.^paramDef.Power_E + N_E.^paramDef.Power_E + paramDef.D));
    diff_E = E_pot - actE;
    GD_E = paramDef.Growth_E .* (diff_E >= 0) + paramDef.Decay_E .* (diff_E < 0);
    k4_incE = GD_E .* diff_E;
    
    % attention process
    Input_A = max(0,S);
    %   N_A = (weights_A_fromA * actA)+ (weights_A_fromE * actE);
    N_A = (weights_A_fromA * actA);
    A_pot = max(0,(Input_A.^paramDef.Power_A) ./ (Input_A.^paramDef.Power_A + N_A.^paramDef.Power_A + paramDef.D));
    diff_A = A_pot - actA;
    GD_A = paramDef.Growth_A .* (diff_A >= 0) + paramDef.Decay_A .* (diff_A < 0);
    k4_incA = GD_A .* diff_A;
    
    % element associability
    k4_incAlpha = paramDef.k_Alpha * (actE - Alpha);
    
    % associative weight change
    Beta_E = paramDef.Beta_E_pos * (k4_incE >= 0) + paramDef.Beta_E_neg * (k4_incE < 0);
    Beta_I = paramDef.Beta_I_pos * (k4_incI >= 0) + paramDef.Beta_I_neg * (k4_incI < 0);
    diffV = ((Beta_E .* k4_incE) - (Beta_I .* k4_incI));
    k4_incV = dV_dt + (paramDef.k_V * ((((diffV.*alph)*alph').*connectivity_E_fromE) - dV_dt));
    
    %final positions of activations and weight changes
    dE_dt = (1/6).*(k1_incE + (2 .* k2_incE) + (2 .* k3_incE) + k4_incE);
    activation_E = activation_E + dE_dt;
    dI_dt = (1/6).*(k1_incI + (2 .* k2_incI) + (2 .* k3_incI) + k4_incI);
    activation_I = activation_I + dI_dt;
    dA_dt = (1/6).*(k1_incA + (2 .* k2_incA) + (2 .* k3_incA) + k4_incA);
    activation_A = activation_A + dA_dt;
    dAlpha_dt = (1/6).*(k1_incAlpha + (2 .* k2_incAlpha) + (2 .* k3_incAlpha) + k4_incAlpha);
    Alpha = Alpha + dAlpha_dt;
    dV_dt = (1/6).*(k1_incV + (2 .* k2_incV) + (2 .* k3_incV) + k4_incV);
    weights_E_fromE = weights_E_fromE + dV_dt;
    
    
    rec_E(:,tick) = activation_E;
    rec_I(:,tick) = activation_I;
    rec_A(:,tick) = activation_A;
end
%% output

if paramDef.fig_V
    final_WE = weights_E_fromE;
    final_WE = cat(1,final_WE,zeros(1,size(final_WE,2),1));
    final_WE = cat(2,final_WE,zeros(size(final_WE,1),1));
    figure
    hV=imagesc(weights_E_fromE);
    weightLabel=[stimDef(:,1)'; cell(size(stimDef(:,1)'))];
    set(get(hV,'Parent'),'XAxisLocation', 'top','XTick',[paramDef.num_elements/2:paramDef.num_elements/2:num_total_elements], 'XTickLabel', weightLabel(:),...
        'YTick',[paramDef.num_elements/2:paramDef.num_elements/2:num_total_elements], 'YTickLabel', weightLabel(:),'YTickMode','manual');
    title('associations between E units')
    xlabel('sending units')
    ylabel ('recipient units')
    colorbar;
end

if paramDef.fig_E
    figure
    hE=imagesc(rec_E);colorbar
    weightLabel=[stimDef(:,1)'; cell(size(stimDef(:,1)'))];
    set(get(hE,'Parent'),'YTick',[paramDef.num_elements/2:paramDef.num_elements/2:num_total_elements], 'YTickLabel', weightLabel(:));
    title('activation of E units')
    xlabel('time (steps)')
    ylabel('units')
    colorbar
end

if paramDef.fig_I
    figure
    hI=imagesc(rec_I);colorbar
    weightLabel=[stimDef(:,1)'; cell(size(stimDef(:,1)'))];
    set(get(hI,'Parent'),'YTick',[paramDef.num_elements/2:paramDef.num_elements/2:num_total_elements], 'YTickLabel', weightLabel(:));
    title('activation of I units')
    xlabel('time (steps)')
    ylabel('units')
end

if paramDef.fig_A
    figure
    hA=imagesc(rec_A);colorbar
    weightLabel=[stimDef(:,1)'; cell(size(stimDef(:,1)'))];
    set(get(hA,'Parent'),'YTick',[paramDef.num_elements/2:paramDef.num_elements/2:num_total_elements], 'YTickLabel', weightLabel(:));
    title('activation of A units')
    xlabel('time (steps)')
    ylabel('units')
end

% CR output
% reshape USactivation, so that we have a nice matrix containing for each trial type the activation in a trial
% I'm sure there is a less brutal way to do this ...perhaps something like this: http://www.mathworks.com.au/matlabcentral/newsreader/view_thread/245285
for pp=1:size(phaseDef,1)
    USactivation=ones(size(trialDef,1),num_TrialsPerPhase(pp))*NaN;
    trials_in_phase=trials(pp,trials(pp,:)>0);
    TrialTypesInPhase=unique(trials_in_phase);
    for tt=1:length(trials_in_phase)
        % find stimulus that should be tested
        USIndex=find(trialDef{trials_in_phase(tt),5}-trialDef{trials_in_phase(tt),4}>0);
        % find ticks that should be tested
        startIndex=paramDef.num_TicksPerTrial*sum(num_TrialsPerPhase(1:pp-1))+ paramDef.num_TicksPerTrial*(tt-1)+trialDef{trials_in_phase(tt),4}(USIndex);
        endIndex=paramDef.num_TicksPerTrial*sum(num_TrialsPerPhase(1:pp-1))+paramDef.num_TicksPerTrial*(tt-1)+trialDef{trials_in_phase(tt),5}(USIndex);
        % calculate mean over elements and ticks and write it in
        % the line in USactivaion matrix that corresponds to trial type
        USactivation(trials_in_phase(tt),tt)=mean(mean(rec_E(paramDef.num_elements*(USIndex-1)+1:paramDef.num_elements*USIndex, startIndex:endIndex),1),2);
    end
    
    % get rid of all the NaN in USactivation
    if all(isnan(USactivation)) %complete phase without any testing
        output{pp}=0;
    else 
        % remove lines containing trial types that were not presented in this pahse, ie. only keep trials that were actually presented in this phase
        USactivation=USactivation(TrialTypesInPhase,:);
        
        % remove activation of "non-presented trials"=NaN, must be loop because
        % otherwise matrix would be reshaped into a vector
        output{pp}=ones(length(TrialTypesInPhase),num_TrialsPerPhase(pp))*NaN;
        for ss=1:length(TrialTypesInPhase)
            % count how often this trial type was presented in this phase
            noTrials=sum(trials_in_phase==(TrialTypesInPhase(ss)));
            
            if all(isnan(USactivation(ss,:))) % if trial type was presented but we don't have any USactivation for it: do nothing
                update=[update ;['No test stimulus for Trial "' trialDef{TrialTypesInPhase(ss),1} '" specified.']];
                set(hOutput,'String',update, 'ListboxTop',length(update));drawnow;
            else % otherwise: write USactivation in the first columns of the output matrix
                output{pp}(ss,1:noTrials)=USactivation(ss,~isnan(USactivation(ss,:)));
            end
        end
    end
    % remove spare columns
    output{pp}=output{pp}(:,~all(isnan(output{pp}),1));
    
    if paramDef.fig_CR
        if ~exist('hCR', 'var'); hCR=figure;end
        subplot(1,size(phaseDef,1),pp)
        if size(output{pp},2)<2
            bar(output{pp})
            set(gca, 'XTickLabel', trialDef(TrialTypesInPhase,1))
        else
            plot(output{pp}');
            legend(trialDef(TrialTypesInPhase,1))
        end
        title(phaseDef{pp,1})
        xlabel('trial no.')
        ylabel(['mean activation of E units of test stimulus'])
    end
end


if paramDef.mat_CR ==1
    filename=uiputfile('output.xls', 'Save activation of test stimulus ');
    if ~isequal(filename,0)
        for pp=1:size(phaseDef,1)
            warning off MATLAB:xlswrite:AddSheet
            xlswrite(filename, [trialDef(unique(trials(pp,trials(pp,:)>0)),1)'; num2cell(output{pp}')], phaseDef{pp,1})
        end
        update=[update ;['Saved data to "' filename '".']];
        set(hOutput,'String',update, 'ListboxTop',length(update));drawnow;
    end
elseif paramDef.mat_CR==2
    uisave('output','output.mat')
    update=[update ;['Saved data as mat-file.']];
    set(hOutput,'String',update, 'ListboxTop',length(update));drawnow;
end
